//+------------------------------------------------------------------+
//|                                                   IchimokuEA.mq5 |
//|                                       Copyright 2022, Nickityuk. |
//|                              https://www.mql5.com/users/nicityuk |
//+------------------------------------------------------------------+
#property copyright "Copyright 2022, Nickityuk."
#property link      "https://www.mql5.com/users/nicityuk"
#property version   "1.00"

#include <Trade\Trade.mqh>
#include <Indicators\Trend.mqh>
#include <Indicators\Indicators.mqh>
#include <Expert\Expert.mqh>

input double MaximumRisk        = 0.02;    // Maximum Risk in percentage
input int    TenkanSen          = 17;
input int    KijunSen           = 26;
input int    SenkouSpanB        = 52;
//---
int         ExtHandleIch=0; //HandleIchimoku
bool        ExtHedging=false;
CTrade      ExtTrade;
CiIchimoku  ExtiIchimoku;
CIndicators ExtIndicators;
#define ExtExpert CExpert     //ExtExpert; 

#define IC_MAGIC 20220529080712
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double TradeSizeOptimized(void)
  {
   double price=0.0;
   double margin=0.0;
//--- select lot size
   if(!SymbolInfoDouble(_Symbol,SYMBOL_ASK,price))
      return(0.0);
   if(!OrderCalcMargin(ORDER_TYPE_BUY,_Symbol,1.0,price,margin))
      return(0.0);
   if(margin<=0.0)
      return(0.0);

   double lot=NormalizeDouble(AccountInfoDouble(ACCOUNT_MARGIN_FREE)*MaximumRisk/margin,2);
//--- normalize and check limits
   double stepvol=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
   lot=stepvol*NormalizeDouble(lot/stepvol,0);

   double minvol=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);
   if(lot<minvol)
      lot=minvol;

   double maxvol=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MAX);
   if(lot>maxvol)
      lot=maxvol;
//--- return trading volume
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open position conditions                               |
//+------------------------------------------------------------------+
void CheckForOpen(void)
  {
   MqlRates rt[2];
//--- go trading only for first ticks of new bar
   if(CopyRates(_Symbol,_Period,0,2,rt)!=2)
     {
      Print("CopyRates of ",_Symbol," failed, no history");
      return;
     }
   if(rt[1].tick_volume>1)
      return;
//--- get current Ichimoku
   double   tk0[1];
   if(CopyBuffer(ExtHandleIch,0,0,1,tk0)!=1)
     {
      Print("CopyBuffer from iIchimoku failed, no data");
      return;
     }
   double   tk1[1];
   if(CopyBuffer(ExtHandleIch,0,1,1,tk1)!=1)
     {
      Print("CopyBuffer from iIchimoku failed, no data");
      return;
     }     
//--- check signals
   ENUM_ORDER_TYPE signal=WRONG_VALUE;

   if(tk1[0]>tk0[0])
      signal=ORDER_TYPE_SELL;    // sell conditions
   else
     {
      if(tk1[0]<tk0[0])
         signal=ORDER_TYPE_BUY;  // buy conditions
     }
//--- additional checking
   if(signal!=WRONG_VALUE)
     {
      if(TerminalInfoInteger(TERMINAL_TRADE_ALLOWED) && Bars(_Symbol,_Period)>100)
         ExtTrade.PositionOpen(_Symbol,signal,TradeSizeOptimized(),
                               SymbolInfoDouble(_Symbol,signal==ORDER_TYPE_SELL ? SYMBOL_BID:SYMBOL_ASK),
                               0,0);
     }
//---
  }
//+------------------------------------------------------------------+
//| Check for close position conditions                              |
//+------------------------------------------------------------------+
void CheckForClose(void)
  {
   MqlRates rt[2];
//--- go trading only for first ticks of new bar
   if(CopyRates(_Symbol,_Period,0,2,rt)!=2)
     {
      Print("CopyRates of ",_Symbol," failed, no history");
      return;
     }
   if(rt[1].tick_volume>1)
      return;
//--- get current Ichimoku
   double   tk0[1];
   if(CopyBuffer(ExtHandleIch,0,0,1,tk0)!=1)
     {
      Print("CopyBuffer from iIchimoku failed, no data");
      return;
     }
   double   tk1[1];
   if(CopyBuffer(ExtHandleIch,0,1,1,tk1)!=1)
     {
      Print("CopyBuffer from iIchimoku failed, no data");
      return;
     }       
//--- positions already selected before
   bool signal=false;
   long type=PositionGetInteger(POSITION_TYPE);

   if(type==(long)POSITION_TYPE_BUY && tk1[0]>tk0[0])
      signal=true;
   if(type==(long)POSITION_TYPE_SELL && tk1[0]<tk0[0])
      signal=true;
//--- additional checking
   if(signal)
     {
      if(TerminalInfoInteger(TERMINAL_TRADE_ALLOWED) && Bars(_Symbol,_Period)>100)
         ExtTrade.PositionClose(_Symbol,5);
     }
//---
  }
//+------------------------------------------------------------------+
//| Position select depending on netting or hedging                  |
//+------------------------------------------------------------------+
bool SelectPosition()
  {
   bool res=false;
//--- check position in Hedging mode
   if(ExtHedging)
     {
      uint total=PositionsTotal();
      for(uint i=0; i<total; i++)
        {
         string position_symbol=PositionGetSymbol(i);
         if(_Symbol==position_symbol && IC_MAGIC==PositionGetInteger(POSITION_MAGIC))
           {
            res=true;
            break;
           }
        }
     }
//--- check position in Netting mode
   else
     {
      if(!PositionSelect(_Symbol))
         return(false);
      else
         return(PositionGetInteger(POSITION_MAGIC)==IC_MAGIC); //---check Magic number
     }
//--- result for Hedging mode
   return(res);
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit(void)
  {
//--- prepare trade class to control positions if hedging mode is active
   ExtHedging=((ENUM_ACCOUNT_MARGIN_MODE)AccountInfoInteger(ACCOUNT_MARGIN_MODE)==ACCOUNT_MARGIN_MODE_RETAIL_HEDGING);
   ExtTrade.SetExpertMagicNumber(IC_MAGIC);
   ExtTrade.SetMarginMode();
   ExtTrade.SetTypeFillingBySymbol(Symbol());
//--- Ichimoku indicator
   ExtHandleIch=iIchimoku(_Symbol,_Period,TenkanSen,KijunSen,SenkouSpanB);
   if(ExtHandleIch==INVALID_HANDLE)
     {
      printf("Error creating Ichimoku indicator");
      return(INIT_FAILED);
     }      
//--- ok
   return(INIT_SUCCEEDED);
  }             
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
//int OnInit()
  //{
//--- create timer
   //EventSetTimer(60);
//---
   //return(INIT_SUCCEEDED);
  //}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- destroy timer
   //EventKillTimer();
  }
//+------------------------------------------------------------------+
//| OnTick handler                                                   |
//+------------------------------------------------------------------+
//void ExtExpert::OnTick(void)
  //{
//--- check process flag
   //if(!m_on_tick_process)
     // return;
//--- updated quotes and indicators
   //if(!Refresh())
     // return;
//--- expert processing
   //Processing();
  //}  
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void Processing()
  {
//---
   if(SelectPosition())
      CheckForClose();
   else
      CheckForOpen();  
//---
  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---
  }
//+------------------------------------------------------------------+
//| Trade function                                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
//---
  }
//+------------------------------------------------------------------+
//| TradeTransaction function                                        |
//+------------------------------------------------------------------+
void OnTradeTransaction(const MqlTradeTransaction& trans,
                        const MqlTradeRequest& request,
                        const MqlTradeResult& result)
  {
//---
  }
//+------------------------------------------------------------------+
//| Tester function                                                  |
//+------------------------------------------------------------------+
double OnTester()
  {
//---
   double ret=0.0;
//---
   return(ret);
  }
//+------------------------------------------------------------------+
//| TesterInit function                                              |
//+------------------------------------------------------------------+
void OnTesterInit()
  {
//---
  }
//+------------------------------------------------------------------+
//| TesterPass function                                              |
//+------------------------------------------------------------------+
void OnTesterPass()
  {
//---
  }
//+------------------------------------------------------------------+
//| TesterDeinit function                                            |
//+------------------------------------------------------------------+
void OnTesterDeinit()
  {
//---
  }
//+------------------------------------------------------------------+
